<?php

declare(strict_types=1);

namespace Drupal\skpr_logs\Logger;

use Drupal\Core\Logger\LogMessageParserInterface;
use Drupal\Core\Logger\RfcLoggerTrait;
use Drupal\Core\Logger\RfcLogLevel;
use Psr\Log\LoggerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Logger for Skpr.
 */
final class JsonLogger implements LoggerInterface {

  use RfcLoggerTrait;

  const SKPR_LOGS_COMPONENT = 'drupal';

  /**
   * Constructs a JsonLogger object.
   */
  public function __construct(
    private readonly LogMessageParserInterface $parser,
    private readonly RequestStack $requestStack,
    private readonly string $stream,
  ) {
  }

  public function log(mixed $level, string|\Stringable $message, array $context = []): void {
    $severity = RfcLogLevel::getLevels()[$level];

    // Populate the message placeholders and then replace them in the message.
    $variables = $this->parser->parseMessagePlaceholders($message, $context);
    $message = $variables !== [] ? \strtr($message, $variables) : $message;

    $data = [
      // Allows development teams to filter by this log type.
      'skpr_component' => self::SKPR_LOGS_COMPONENT,
      'timestamp' => (new \DateTimeImmutable('@' . $context['timestamp']))
        ->format('Y-m-d\TH:i:s'),
      'severity' => $severity,
      'message' => \strip_tags($message),
    ];

    $request = $this->requestStack->getCurrentRequest();
    if (!\is_null($request)) {
      // Allows development teams to trace interactions.
      $data['request_id'] = $request->headers->get('X_REQUEST_ID', '');
    }

    $event = \json_encode($data);

    $output = \fopen($this->stream, 'w');
    if ($output === FALSE) {
      throw new \Exception(\sprintf('Failed to get stream: %s', $this->stream));
    }
    \fwrite($output, $event . "\n");
    \fclose($output);
  }

}
