<?php

declare(strict_types=1);

namespace Drupal\Tests\skpr_logs\Unit;

use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\Core\Logger\LogMessageParserInterface;
use Drupal\Core\Logger\RfcLogLevel;
use Drupal\skpr_logs\Logger\JsonLogger;
use Drupal\Tests\UnitTestCase;
use org\bovigo\vfs\vfsStream;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Tests the JsonLogger class.
 *
 * @coversDefaultClass \Drupal\skpr_logs\Logger\JsonLogger
 * @group skpr_logs
 */
class JsonLoggerTest extends UnitTestCase {

  protected function setUp(): void {
    parent::setUp();
    $container = new ContainerBuilder();
    $container->set('string_translation', $this->getStringTranslationStub());
    \Drupal::setContainer($container);
  }

  /**
   * Tests the log method.
   *
   * @covers ::log
   */
  public function testLog(): void {
    $parser = $this->createMock(LogMessageParserInterface::class);
    $parser->method('parseMessagePlaceholders')->willReturn([]);

    $request = new Request();
    $request->headers->set('X_REQUEST_ID', '1234567890');

    $requestStack = $this->createMock(RequestStack::class);
    $requestStack->method('getCurrentRequest')->willReturn($request);

    $vfs = vfsStream::setup();
    $logs = vfsStream::newFile('logs.json')->at($vfs);
    $logger = new JsonLogger($parser, $requestStack, $logs->url());

    $message = 'Test message';
    $context = ['timestamp' => 1234567890];
    $logger->log(RfcLogLevel::INFO, $message, $context);

    $content = $logs->getContent();
    static::assertEquals(
      $content,
      '{"skpr_component":"drupal","timestamp":"2009-02-13T23:31:30","severity":"Info","message":"Test message","request_id":"1234567890"}' . "\n",
    );
  }

}
